//
//  IUdbAuthDL.h
//  udbauth
//
//  Created by fq on 2016/12/12.
//  Copyright © 2016年 IT. All rights reserved.
//  

#ifndef __yyproto__iudbauth_dl__
#define __yyproto__iudbauth_dl__

#include <stdio.h>
#include <string>
#include <string.h>

#ifdef ANDROID
#include <malloc.h>
#endif

using namespace std;

#ifdef _WIN32
#ifdef _MSC_VER
typedef __int32 int32_t;
typedef unsigned __int32 uint32_t;
typedef __int64 int64_t;
typedef unsigned __int64 uint64_t;
#endif
#endif


#ifdef _WIN32
#ifndef _DLL_UDB
#define DLL_UDB_API extern "C" __declspec(dllexport)
#else
#define DLL_UDB_API __declspec(dllimport)
#endif
#endif

#ifndef UDBEXPORT
#define UDBEXPORT  __attribute__ ((visibility ("default")))
#endif

#define AUTH_DECL_PROP_DL(T, P)	\
private: T	m_##P;	\
public: inline T get_prop_##P() const { return m_##P;}	\
public: inline void set_prop_##P(const T& value) { m_##P = value;}


namespace udbauth
{
  class UDBEXPORT UdbString {
  public:
    UdbString() : _pdata(NULL), _len(0) {}
    UdbString(const char* pdata) : _pdata(NULL), _len(0) {
      this->CopyCString(pdata, strlen(pdata));
    }
    UdbString(const char* pdata, const size_t n) : _pdata(NULL), _len(0) {
      this->CopyCString(pdata, n);
    }
    UdbString(const UdbString &str) : _pdata(NULL), _len(0) {
      this->CopyCString(str._pdata, str._len);
    }
    UdbString(const string &str) : _pdata(NULL), _len(0) {
      this->CopyCString(str.c_str(), str.length());
    }
    ~UdbString() {
      clear();
    }
    UdbString& operator =(const UdbString &udbstr) {
      this->CopyCString(udbstr._pdata, udbstr._len);
      return *this;
    }
    string str() const {
      return string(_pdata, _len);
    }
    const char* c_str() const {
      return _pdata;
    }
    size_t size() const {
      return _len;
    }
    bool empty() const {
      if (_pdata == NULL || _len == 0) {
        return true;
      }
      return false;
    }
    void clear() {
      if (_pdata) {
        free(_pdata);
      }
      _pdata = NULL;
      _len = 0;
    }
  private:
    void CopyCString(const char* psrc, const size_t n) {
      if (psrc == NULL && n == 0) {
        this->clear();
      }
      if (_pdata) {
        _pdata = (char*)realloc(_pdata, n);
      } else {
        _pdata = (char*)malloc(n);
      }
      if (_pdata) {
        memcpy(_pdata, psrc, n);
        _len = n;
      }
    }
  public:
    char *_pdata;
    size_t _len;
  };
  /**
   *  外部给udbauth的接口
   */
  struct UDBEXPORT IUdbAuthWatcherDL
  {
    /**
     *  发送到后端服务器的请求
     *
     *  @param uri    uri
     *  @param yypack yy二进制请求消息，此消息不包含10字节包头。10字节包头是：4字节len＋4字节uri＋2字节rescode
     */
    virtual void sendToServer(uint32_t uri, const char* pyypack, const size_t n) = 0;
    
    /**
     *  发送到UI的响应
     *
     *  @param type 请求类型（auth->OP_CMD）
     *  @param json 服务端返回数据（json格式）
     */
    virtual void sendToUI(uint32_t type, const char* pjson, const size_t n) = 0;
    
    
    /**
     *  发送日志给调用方，由调用方打印
     *
     *  @param log 日志内容
     */
    virtual void sendLog(const char* plog, const size_t n) = 0;
    
    /**
     *  运行虚拟机代码
     *
     *  @param code 代码
     *
     *  PC YY 或 PC LoginSDK使用时，不要实现此回调。直接调用外挂库lib。
     *  YYSDK使用时，需要从C++层实现此回调。
     *  LoginSDK ios使用时， oc层实现回调。
     *  LoginSDK android使用时， java层实现回调。
     */
    virtual UdbString runCode(const char *runcode, const size_t n, const bool isAnony) = 0;
  };
  
  
  struct UDBEXPORT AuthResultDL {
    /*
    	uiaction取值含义：
    	0:登陆成功
    	1:登陆失败，UI弹提示
    	2:需要下一步验证，具体验证看strategy
    	3:凭证失效，UI删除凭证，要求重新输入密码
    	4:动态校验错误（图片、滑动、短信、手机令牌、硬件令牌）
    	5:服务端还没收到上行短信，若用户继续停留在等待界面，应用要再发一个check_smsup去检测上行短信
    	*/
    int	   uiAction;
    int    errCode;     //errCode是UDB登录服务返回的业务码，数量较多，一般不需要关注。
    UdbString	account; //登录时所使用的账号，可以是 手机,邮箱,通行证,YY号
    uint64_t    uid;
    uint64_t	yyid;
    UdbString  passport;
    UdbString  password;
    int	   isAnony;   //0未知或初始值, 1匿名登录,2账号登录
    int    status;   //0 未知状态 ， 1 发出登录请求  ， 2 二次验证 ， 3 登录完成， 4 服务器超时
    
    AuthResultDL():uiAction(-1),errCode(-1),uid(0),yyid(0),passport(""),password(""),isAnony(0),status(0){}
  };
  
  struct UDBEXPORT IAuthResultWatcherDL
  {
    virtual void notifyResult(const struct AuthResultDL & result) = 0;
  };
    
  struct UDBEXPORT IAuthReportLogWatcherDL
  {
    virtual void ReportLog(int log_type, const char* plog, const size_t n) = 0;
  };
  
  class UDBEXPORT UdbInfoDL {
  public:
    UdbInfoDL():m_deviceName(""),m_deviceId(""),m_deviceInfo(""),m_imei(""),m_macAddr(""),m_platform(""),m_lcid(""),
    m_mnc(""),m_mcc(""),m_systemVer(""),m_netType(""),m_appId(""),m_appKey(""),m_appVer(""),m_terminalType(0),m_enableAnony(false)
    {}
    //手机硬件信息
    AUTH_DECL_PROP_DL(UdbString, deviceName)    //设备名称。手机型号：MINOTELTE,IPHONE 6；PC的计算机名：YY-20141112WEZU
    AUTH_DECL_PROP_DL(UdbString, deviceId)      //从黄俊的device库获取的设备ID
    AUTH_DECL_PROP_DL(UdbString, deviceInfo)    //从黄俊的device库获取的设备信息
    AUTH_DECL_PROP_DL(UdbString, imei)          //手机IMEI
    AUTH_DECL_PROP_DL(UdbString, macAddr)       //mac地址
    AUTH_DECL_PROP_DL(UdbString, platform)      //暂无用
    AUTH_DECL_PROP_DL(UdbString, lcid)          //语言标志，其中2052简体中文
    AUTH_DECL_PROP_DL(UdbString, mnc)           //手机mnc
    AUTH_DECL_PROP_DL(UdbString, mcc)           //手机mcc
    AUTH_DECL_PROP_DL(UdbString, systemVer)     //操作系和版本，例如：Windows10.1   ios9.1  android5.0
    AUTH_DECL_PROP_DL(UdbString, netType)       //暂无用
    //业务使用方信息
    AUTH_DECL_PROP_DL(UdbString, appId)         //udb分配的APPID
    AUTH_DECL_PROP_DL(UdbString, appKey)        //udb分配的APPKEY
    AUTH_DECL_PROP_DL(UdbString, appVer)        //业务方的APP版本号
    AUTH_DECL_PROP_DL(uint64_t, terminalType)     //ap分配的terminalType
    //是否开启匿名登陆
    AUTH_DECL_PROP_DL(bool,  enableAnony)      //是否开启匿名功能。开启后auth会主动发起匿名请求，获取匿名uid。
  };
  
  
  ///调用方通知udbauth网络状态，在setNetStatus中设置
  struct UDBEXPORT UdbAuthEnum{
    enum {
      UDBAUTH_NETSTATUS_CLOSE = 1,
      UDBAUTH_NETSTATUS_CONNECTING,
      UDBAUTH_NETSTATUS_READY,
      UDBAUTH_NETSTATUS_RECONNECTING,
      UDBAUTH_NETSTATUS_ERROR,
    };
    enum CarrierType {
      NET_CLOSE = 1,
      NET_UNKNOW,		  //未知的网络提供商
      NET_LAN,   //有线网络
      NET_WIFI,  //wifi无线网络
      NET_UNION_2G, //联通2G网络
      NET_UNION_3G, //联通3G网络
      NET_UNION_4G, //联通4G网络
      NET_MOBILE_2G, //移动2G网络
      NET_MOBILE_3G, //移动3G网络
      NET_MOBILE_4G, //移动4G网络
      NET_TELECOM_2G, //电信2G网络
      NET_TELECOM_3G, //电信3G网络
      NET_TELECOM_4G, //电信4G网络
    };
    enum LogType //日志上报类型
    {
        UDB_REQUEST_LOG = 1, //请求日志
        UDB_TIMEOUT_LOG,    //超时日志
    };
  };
  
  class UDBEXPORT IUdbAuthDL
  {
  public:
    /**
     *  获取实现ISignalAuth的实例
     *
     *  @return 实现ISignalAuth的实例
     */
    static IUdbAuthDL * getInstance();

  public:
    
    /**
     *  初始化app相关信息
     *
     *  @param auththread 是否开启auth自己的线程。0不开启，1开启
     *  @param version   0:YYSDK， 1:Mobile LoginSDK  2:PCYY  3:PC LoginSDK
     */
    virtual void init(const int auththread = 0, const int version = 0, const char *filePath = NULL) = 0;
    
    /**
     *  销毁在init中初始化的资源，例如开启的独立线程、new的内部变量
     *  destroy后可以重新init
     */
    virtual void destroy() = 0;
    
    /**
     *  设置sdk内部文件存储路径，此接口和init里的filePath是一样效果的。
     */
    virtual void setFilePath(const char* filePath) = 0;
    
    /**
     *  设置是否开启匿名登陆功能
     *
     *  @param disable 是否开启匿名登录功能
     *
     *  12月01日修正，匿名登录不使用此接口设定
     */
    virtual void setUdbInfo(const UdbInfoDL& udbinfo) = 0;
    
    /**
     *  业务方app使用此接口，增加业务方后台校验需要用到的appid
     *
     *  @param appid  增加这个appid后，能从udb服务器获取的ticket中包含这个appid的token
     */
    virtual void insertVerifyAppid(const char* appid) = 0;
    
    /**
     *  设置是否开启匿名登陆功能
     *
     *  @param enable 是否开启匿名登录功能
     */
    virtual void setAnonymous(bool enable) = 0;
    
    /**
     *  注册回调接口，回调提供sendData与sendEvent的接口
     *
     *  @param watcher Auth模块与login交互回调接口实现类
     */
    virtual void watch(IUdbAuthWatcherDL* watcher) = 0;
    /**
     *  反注册回调接口
     *
     *  @param watcher
     */
    virtual void revoke(IUdbAuthWatcherDL* watcher) = 0;
    
    
    /**
     *  注册回调接口，回调提供IAuthResultWatcher的接口
     *
     *  @param watcher
     */
    virtual void watchResult(IAuthResultWatcherDL* watcher) = 0;
    /**
     *  反注册回调接口
     *
     *  @param watcher
     */
    virtual void revokeResult(IAuthResultWatcherDL* watcher) = 0;
    
    
    /**
     *  预判断这个request是否一个新的账号登录
     *
     *  @param request 请求数据（json格式）
     */
    virtual bool isStartLogin(const char* request) = 0;
    
    /**
     *  处理UI的发起的请求
     *
     *  @param request 请求数据（json格式）
     */
    virtual void sendRequest(const char* request, const size_t n) = 0;
    
    /**
     *  处理后台服务器下发的包
     *
     *  @param data 服务端返回的YY二进制消息
     */
    virtual void handleResponse(const char* data, const size_t n) = 0;
    virtual void handleResponse(uint32_t uri, const char* data, const size_t n) = 0;
    
    /**
     *  同步auth的内部时间。外部调用方若有pingpong，需要至少每10分钟设置一次这个时间
     *
     *  @param time
     */
    virtual void syncServerTime(uint64_t servertime) = 0;
    
    /**
     *  接受连接层事件
     *
     *  @param netStatus 调用方通知udbauth网络状态
     */
    virtual void setNetStatus(int netStatus) = 0;
      
    /**
     *  login模块登录ap票据失效，重新到udb登录更新票据，重新校验所需数据保存在auth内部
     */
    virtual void reAuth() = 0;
    
    /**
     *  用户注销接口
     */
    virtual void logout() = 0;
    
    /**
     *  有使用方主动发起，获取日志
     *
     *  @return 日志信息（json格式）
     */
    virtual UdbString getLog() = 0;
    
    /**
     *  原来的uauth库功能合拼到auth里。解开ticket，获取token
     *
     *  @param appid  appid
     *  @param encode 0-二进制，1-base64，2-base64+url
     *
     *  @return token
     */
    virtual UdbString getToken(const char* appid, const uint32_t encode, const char* productId = "") = 0;
    
    
    /**
     *  原来的uauth库功能合拼到auth里。解开ticket，获取token
     *
     *  @param passport  passport
     *  @param appid  appid
     *  @param encode 0-二进制，1-base64，2-base64+url
     *
     *  @return token
     */
    virtual UdbString getTokenByPassport(const char* passport, const char* appid, uint32_t encode) = 0;
    
    /**
     *  获取webtoken（appid为5060，编码方式为base64+url）
     *
     *  @return webtoken
     */
    virtual UdbString getWebToken() = 0;
    
    /**
     *  获取yycookies
     *
     *  @return yycookies
     */
    virtual UdbString getYYCookies() = 0;
    
    /**
     *  获取OTP
     *
     *  @return OTP
     */
    virtual UdbString getOTP(const char* appid) = 0;
    
    /**
     *  获取最后一次登录的结果
     *
     *  @return AuthResult
     */
    virtual const AuthResultDL getAuthResult() = 0;
    /**
     *  获取种子票据
     *
     *  @return ticket
     */
    virtual UdbString getTicket() = 0;
    
    /**
     *  获取保存在本地文件的credit
     *
     *  @return credit
     */
    virtual UdbString getCredit(const uint64_t uid) = 0;
    /**
     *  清除保存在本地文件的credit
     *
     *  @param uid   当uid为0时清除所有uid的credit。uid非0则清除指定uid的credit
     */
    virtual void clearCredit(uint64_t uid) = 0;
    /**
     *  外部设置定时器调用
     *
     *  @param interval 定时器的时间间隔，单位ms，建议100ms
     */
    virtual void onTimerDo(int interval=100) = 0;
    
    /**
     *  将明文密码转换成SHA1，此接口暂时没用
     *
     *  @param password 明文密码
     */
    virtual UdbString getSHA1Password(const char* password) = 0;
    
    /**
     *  获取udb的web页面参数（例如：二维码页面、注册页面）所需的参数
     */
    virtual UdbString getDeviceData() = 0;
    
    /**
     *  解码二维码登录参数
     */
    virtual UdbString getQrLoginData(const char* data, const size_t n) = 0;
    
    /**
     *  设置运营商类型。取值含义参照CarrierType
     */
    virtual void setCarrierType(uint32_t type) = 0;
    
    /**
     *  版本号
     */
    virtual UdbString getVersion() = 0;
    
    /*
     *  同步通用接口
     */
    virtual UdbString udbAuthFuncation(const char* json, const size_t n) = 0;
      
    /*
    *日志上报接口注册
    */
    virtual void watchLogReporter(IAuthReportLogWatcherDL * watcher) = 0;
    
  };
  
}

#ifdef _WIN32
	DLL_UDB_API udbauth::IUdbAuthDL* _cdecl GetIUDBAuthDL(void);
	typedef udbauth::IUdbAuthDL* (__cdecl *PGetIUDBAuthDL)(void);
#endif

#ifdef ANDROID
extern "C"  udbauth::IUdbAuthDL*   getIUdbAuthDL(void);
typedef  udbauth::IUdbAuthDL* (*GET_IUDBAUTHDL_FUNC)(void);
#endif

#endif /* __yyproto__iudbauth_dl__ */
